
/*	FL02.C 	*/

/*

	FLIST Utility  
	
	Copyright (C) JXL Systems Inc. 1987.  All rights reserved.

	Source file 2 of 3
	
*/

#include "syshdrs.h"    /* standard compiler headers */
#include "textlib.h"	/* the TextLib header files */

#include "flist.h"		/* the FLIST header file */


/***********************************************************************/
/*                                                                     */
/*	This source file contains functions for:                           */
/*       . Selecting an action from the main FLIST menu                */
/*       . Selecting a directory to process                            */
/*       . Selecting a file within the selected directory              */
/*       . Specifying the action to be taken upon that file            */
/*                                                                     */
/***********************************************************************/


/***********************************************************************/
/*                                                                     */
/*	These are the MAIN-MENU-handling functions                         */
/*                                                                     */
/***********************************************************************/

mmentry()	/* menu entry (at start of FLIST processing) */

{

	MenuMount(); /* Display the main menu */
	mminstr();	/* FLIST subroutine to display instructions on screen */
	return(1);
	
}

mmres()  /* Menu resumed after a selected action terminates */

{

	mminstr();		/* FLIST subroutine to display instructions on screen */
	return(1);
	
}


mm()	/* menu service */

{

	while (1)   /* until user makes a selection */
	{
	
		kpfunret = MenuInput(ipmain);	/* TextLib -- get menu choice */
		
		if (kpfunret >= -1)
			break;	/* values < -1 are F-keys; not used in this menu */
	
	}
	
	if (kpfunret == -1)  	/* ESC */
		return(0);          /* return code of 0 terminates loop in main() */
	
	/*	kpfunret is positive */
	
	switch (MenuSelection(mainmenu)) 	/* user's selection */
	{
	
	case	MainChooseDirectory:

		StackAction("dirname");
		break;
		
	case	MainQuit:

		return(0);	/* zero return-code causes FLIST to terminate */
		
	}
	
	return(1);
	
}

mminstr()		/* display instructions */

{

	Display(FunctionDisplayWindow," MAIN MENU \x18\xff\xc0");
	clrstat();	/* clear fields at top right of screen */
	menuinst();	/* display menu instructions */
	return(1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are the DIRECTORY-NAME-SELECTION functions                   */
/*                                                                     */
/***********************************************************************/


dn()				/* action mainline */

{

int	 not_valid;  
char *pathname;
int  pnlen;
char far *farptr;
int name_start_pos;
int cflag;


	not_valid = 1;	/* presume user's input to be not valid */
	
	do				/* receive input until it is valid */
	{

		StInput(kcode);                /* get user's input */
		
		StInputOrEditDone(pathname,pnlen); /* capture data, length */
		
		/*
		Processing at this point depends on the value in 
		kcode (the code for the keypress that
		ended the input operation).  Note that TextLib has
		its own method of encoding keypresses.
		*/
		
		switch (kcode) 	/* user's terminating keypress */
		{
		
		case	ESC:	/* user wants to forget it */
			
			UnstackAction();    	/* TextLib -- unstack this action */
			not_valid = 0;	/* mark as valid input */
			break;

		case	ENTER:		/* user has data for us */

			strcpy(pathn,pathname);	/* copy to external area */
			name_start_pos = 0;
			if (pnlen) 
			{
				if (pathn[0] == '.') 	/*	handle . and .. */
				{
					++name_start_pos;
					if ((pnlen >= 2) && (pathn[1] == '.'))
						++name_start_pos;
				}
			}	
			if (! strrchr(&(pathn[name_start_pos]),'.'))  
			{
				if (pnlen && pathn[pnlen - 1] != '\\')
					strcat(pathn,"\\");
				strcat(pathn,"*.*");	/* append *.* */
			}

			/* check validity of specified directory */

			/*	Get pointer to DOS DTA */
			
			getdtptr();

			/*	See if the chosen directory has any files */
				
			inregs.h.ah = 0x4e;	/* DOS -- Search for first match */
			inregs.x.cx = 0;	/* Normal files only */
			farptr = (char far *)&(pathn[0]);
			inregs.x.dx = FP_OFF(farptr);	/* offset of pathname */
			segregs.ds = FP_SEG(farptr);	/* segment of pathname */

			/*	Note: Lattice intdosx returns flags, other
				compilers' intdosx returns AX. */

			cflag = intdosx(&inregs,&outregs,&segregs);	/* DOS call */

#ifdef	LATTC3
			if (cflag & 0x01) 
			{
#else
			if (outregs.x.cflag)  /* if error */
			{
#endif
				Display(DirectoryName_2,
				  "Invalid Path or Empty Directory.\xff\xc0");
				Display(DirectoryName_3,
				  "  Press any key to continue...\xff\xc0");
				if (! getch())
					getch();
				Clear(DirectoryName_2);
				Clear(DirectoryName_3);
			}
				
			else 	/* valid, nonempty directory */
			{

				strcpy(fullfn,pathn);
				fnsplit(fullfn,drive,dir,fnm,ext);
				strcpy(pathpart,drive);
				strcat(pathpart,dir);
			
				/* \xff\xa0\x82 makes text display in green */
					
				Display(DirectoryName_2,
					   "\xff\xa0\x82Reading Directory...\xff\xc0");
					
				/* invoke FLIST function to build a directory "image" */
					
				blddir();
						
				/*
				Stack the next action, which is the display of
				the directory list itself.
				*/
					
				StackAction("dirlist");
						
				not_valid = 0;	/* mark as valid input */
			}
				
			break;
					
		}	/* end of switch on kcode value */
		
	}
	
	while (not_valid);

	return(1);
	
}


/*

	Function DNINIT: 
	Common initialization for directory-name procedure.
	Writes instructions on bottom of screen.
	Brings up and initializes the directory-name popup.
	
*/

dninit()

{

	Display(InstructionsWindow_1,
	"Type a directory-name as you would for a DOS \"dir\" command,\xff\xc0");
	Display(InstructionsWindow_2,
		" then press ENTER.\xff\xc0");
	Display(InstructionsWindow_3,
		"OR ... press ESC to return to main menu.\xff\xc0");

	/*
	Clear the status/error fields at the
	top-right of the screen
	*/

	clrstat();
		
	UpPopup(DirectoryNamePopup);
	Display(DirectoryName_1,
		"Directory name:\xff\xc0");

	return(1);
	
}

/*

	Function DNEND:  
	Common termination for directory-name procedure.
	Takes down the directory-name popup.
	
*/

dnend()

{
		
	DownPopup(DirectoryNamePopup);
	return(1);
	
}

/*	Function BLDDIR:  builds a directory list (of FLIST structures) */

blddir()

{

int	i;
int	namelen;
int extlen;
int totlen;
char *ext;
char *dot;
int cflag;

	/*	If there is a previous list, free it */
	
	if (numfiles) 
	{
	
		for (i = 0; i < numfiles; ++i) 
		{
			selfile = *(filevec + i);	/* pointer to filedesc struct */
			free((char *)selfile);
		}
		free((char *)filevec);			/* free the vector-list */
		
		numfiles = 0;
		filelist = 0;
		filevec = 0;
		
	}
	
	/*	At entry, the first file description is already in the DTA */
	
	while (1) 	/* build list */
	{

		if (! (selfile = (struct filedesc *)malloc(sizeof(struct filedesc))))
			outofmem();
			
		selfile -> next = filelist;	/* pointer to (prev) file in direc */
		filelist = selfile;			/* set this as last */
		
		/* break the filename into a name and an extension */
		
		totlen = strlen(dtaptr -> nameext);	/* total length name + ext + . */
		
		dot = memchr(dtaptr -> nameext,'.',totlen); /* get position of . */
		if (dot) 	/* if a dot is present */
		{
			namelen = dot - dtaptr -> nameext;	/* len of name */
			if ((extlen = totlen - (namelen + 1)) > 0)
				ext = dot + 1;	/* pointer to extension */
			else
				ext = 0;	/* filename ends with a dot (??) */
		}
		else		/* no dot is present */
		{
			namelen = totlen;
			extlen = 0;
			ext = 0;
		}
		
		memset(selfile -> name,' ',8);	/* blank name, ext fields */
		memset(selfile -> ext,' ',3);
		selfile -> name[8] = 0;	/* null string terminators */
		selfile -> ext[3] = 0;
		movmem(dtaptr -> nameext,selfile -> name,namelen); /* name */
		if (ext)
			movmem(ext,selfile -> ext,extlen);	/* extension */
			
		/*	Copy the rest of the file information */
		
		selfile -> date = dtaptr -> date;
		selfile -> time = dtaptr -> time;
		selfile -> attr = dtaptr -> attr;
		(selfile -> size).sizeint[0] = dtaptr -> size_lo;
		(selfile -> size).sizeint[1] = dtaptr -> size_hi;
		
		/*	Bump count of file descriptors built */
		
		++numfiles;
		
		/*	Done building descriptor.  Get next directory entry */
		
		inregs.h.ah = 0x4f;	/* DOS -- continue directory search */

		/*	Note: Lattice intdos() returns flags.  Other compilers'
		intdos() returns AX.  */

		cflag = intdos(&inregs,&outregs);

#ifdef LATTC3
		if (cflag & 0x01)
#else
		if (outregs.x.cflag)	/* if no more files */
#endif
			break;
			
	}

	/* Now build a vector list to the descriptors */
	
	/* The sequence will be the order of the directory entries as read */
	
	/* get storage for the array */

	if (! (filevec = 
		   (struct filedesc **)malloc(numfiles * sizeof(struct filedesc *))))
		outofmem();
		
		
	/* fill in the array back-to-front since the structs are stacked LIFO */
	
	for (i = numfiles - 1, selfile = filelist;
		 selfile;
		 --i, selfile = selfile -> next) 
		*(filevec + i) = selfile;	/* set vector */
		
	return(1);	/* all done */
	
}

/***********************************************************************/
/*                                                                     */
/*	These are FILE-SELECTION functions                                 */
/*                                                                     */
/***********************************************************************/


dlentry()	/* entry to file-list */

{

	dlinit();	/* general initialization */
	return(1);
	
}

dlres() /* File-list resumes after subsidiary action terminates */

{

	/*
	Use this TextLib function to select page "ad00a", since
	the screen area occupied by this page is at times occupied by
	other pages.
	*/
	
	SelectPage(DirectoryDisplayWindow);
	
	/*
	Use this TextLib function to restore the video image
	of page "ad00a" (so that it does not have to be rebuilt). The
	image was saved in FLIST function dlsusp() when this action
	was suspended. 
	*/
	
	RestorePageVideo(DirectoryDisplayWindow);

	if (sortflag)   /* if directory was just sorted */
		dlinit();	/* do a full re-initialization */

	else
		dlinstr();        	/* display the instructions */
	
	return(1);
	
}


dl()	/* function in which user selects a file to process */

{

char selname[50];

	while (1) 
	{
	
		sortflag = 0;	/* presume directory will not be sorted */
	
		kpfunret = MenuInput(NULL);  /* TextLib -- get menu selection */
		
		if (kpfunret == -12) 	/* F2 return-code */
		{
			StackAction("sortmenu"); /* user wants to sort the list */
			return(1);
		}

		if (kpfunret == -13)    /* F3 return-code */
			prtdirec();			/* print the directory-listing */
		
		if (kpfunret == -14) 	/* F4 return-code */
		{
			StackAction("creatnew"); /* create new file via editor */
			cn();				/* call service function directly */
			UnstackAction();		/* unstack not self but 'creatnew' */
			if (cn_ok)  		/* if user wants to go forward */
				StackAction("edit");
			return(1);
		}
			
		if (kpfunret >= -1)
			break;	/* excludes other F keys */
			
	}	
	
	if (kpfunret == -1) 	/* ESC */
	{
		UnstackAction();    	/* TextLib -- unstack this action */
		return(1);			/* return to main loop */
	}
	
	/*
	kpfunret is positive.
	Therefore, the user has selected a file.
	We will translate the select-code from the menu
	into a pointer to a file-descriptor, and then
	stack Action "dirmenu" (file-action menu) on top
	of this Action.
	*/

	selfile = *(filevec + (MenuSelection(dirlist) - 1)); /* file descriptor */

	StackAction("dirmenu");

	return(1);
	
}


dlsusp() /* a subsidiary action is about to begin */

{

	/*
	Use this TextLib function to save video page "ad00a" so that
	when this action resumes it can be restored instead of having to
	be rebuilt.
	*/
	
	SavePageVideo(DirectoryDisplayWindow);
	return(1);
	
}


dlexit()	/* when action exits */

{

	Clear(DirectoryDisplayWindow);	/* clear the directory display */
	return(1);
	
}


dlinstr()		/* put instructions on screen */

{

char actid[80];
	
	Display(InstructionsWindow_1,
"Use \x1b\x1a \x18\x19 to move cursor and/or scroll the display\xff\xc0");
	Display(InstructionsWindow_2,
"Press ENTER to select highlighted file\xff\xc0");
	Display(InstructionsWindow_3,
"F2 = sort listing, F3 = print dir, F4 = create new file, ESC = exit\xff\xc0");
	
	/*	Clear the status lines at the top right of the screen */

	clrstat();
	
	/*	If there are too many files to be onscreen at once, so indicate */
	
	if ((((numfiles - 1) / 4) + 1) > HeightOfPage("ad00a"))
		Display(StatusWindow_1,
		  " File list vertically scrollable\xff\xc0");
		
	/*	Display the identification of the action */
	
	strcpy(actid," Listing of Directory: ");
	strcat(actid,pathn);
	strcat(actid,"\xff\xc0");
	Display(FunctionDisplayWindow,actid);

	return(1);
	
}

dlinit()	/* initialization for this menu */

{

int i;

	/*
	Use this TextLib function to select page "ad00a", since
	the screen area occupied by this page is also from time to time
	occupied by other pages.
	*/
	
	SelectPage(DirectoryDisplayWindow);

	/*
	Use the TextLib function MenuReInit() to reset the
	"menu" (i.e., the list of files in the selected directory)
	so that it may be dynamically rebuilt from the
	stored list of files in the selected directory.
	*/

	MenuReInit(dirlist,DirectoryDisplayWindow,4);

	Clear(DirectoryDisplayWindow);	/* clear the window */
		
	/*
	Now we configure the body of the menu by making a call on
	the TextLib function MenuItem() for each file descriptor.
	The order of the calls is determined by the sequence of
	the pointer-array "filevec", which is set by the directory-read
	and directory-sort routines.
	
	Note the parameter "i + 1" in the MenuItem() call.  This establishes
	a select-code to be returned by the TextLib menu-handling
	function when the user selects a file.  This code will be a
	one-relative index to "filevec", the array of pointers to
	file-descriptors.  This is how we will identify the file the
	user wants when he selects it from the menu.
	
	The third (zero) parameter in the MenuItem call indicates that
	selection may be made from this menu by cursoring only; there is
	no key that can be typed to indicate a selection.
	
	The last two MenuItem parameters (2,2) indicate that there is to be
	a padding of two spaces each before and after the prompt (which is
	the filename).  This padding is not reverse-video'd when the item
	is cursored.
	*/
	
	for (i = 0; i < numfiles; ++i) 
	{
		
		selfile = *(filevec + i);	/* get the file descriptor */
		
		movmem(selfile -> name,fmenuent.name,8); /* copy the name */
		movmem(selfile -> ext,fmenuent.ext,3); /* and the extension */
		
		MenuItem(dirlist,(char *)&fmenuent,0,i + 1,2,2);
			
	}
	
	/*
	Now issue TextLib function MenuMount() to display the menu and
	position the cursor at the first entry.
	*/
	
	MenuMount();
		
	/*	Display the instructions */
	
	dlinstr();
		
	return(1);
	
}

/*	Function to print the directory-listing */

prtdirec()

{

int	i;
int  printer;
char listhdr[100];

	if ((printer = open("PRN",O_WRONLY | O_TEXT)) == -1) 
	{
		FastMsg("a","Error starting the printout");
		return;
	}
	
	criterr = 0;			/* reset critical-error flag */

	/* \xff\x82 is a "Blink" video-attribute modifier */

	Display(StatusWindow_1,"\xff\x82 Printing...");

	sprintf(listhdr,"\nLISTING OF DIRECTORY: %s\n\n",pathn);
	putpr(listhdr,printer);
	putpr((char *)&filephdr,printer);
	
	for (i = 0; i < numfiles; ++i) 
	{
	
		selfile = *(filevec + i); /* ptr to file-descriptor structure */
		
		movmem(selfile -> name,fileprt.name,8);
		movmem(selfile -> ext,fileprt.ext,3);
		fileprt.arc = (selfile -> attr) & 0x20 ? 'Y' : 'N';
		fileprt.rdonly = (selfile -> attr) & 0x01 ? 'Y' : 'N';

		getfldat();	/* get file date/time info */

		fmtdt();	/* format file date */
		movmem(wkarea,fileprt.date,8);
		fmttm();	/* file time */
		movmem(wkarea,fileprt.time,6);
		fmtsize();	/* file size */
		movmem(wkarea,fileprt.size,8);

		putpr((char *)&fileprt,printer);
		
	}

	closeprt(printer);

	Clear(StatusWindow_1);

}

putpr(string,handle)		/* do writes to printer */

char *string;
int handle;  

{

int i;
int len;

	len = strlen(string);
	
	for (i = 0; i < len; ++i)
	{

		for (;;)				/* till a char is successfully written */
		{
			write(handle,&(string[i]),1);
			if (! criterr)	/* x_int24h sets this flag if critical err */
				break;
			prtnr();	/* display not-ready message */
		}	
		
	}
	
}

prtnr()						/* printer not ready */

{

	FastMsg("a","Printer not ready");
	criterr = 0;
	
}

closeprt(handle)				/* close a printout */

int handle;   

{

	putpr("\n* * * END OF LISTING * * *\n\x0c",handle);

	for (;;)		/* until close is successful */
	{
		close(handle);
		if (! criterr)
			break;
		prtnr();
	}
	
}

/***********************************************************************/
/*                                                                     */
/*	These are FILE-ACTION-SELECT functions                             */
/*                                                                     */
/***********************************************************************/

dientry()	/* entry to menu listing possible actions */

{

char selname[50];

	menuinst();		/* display instructions */
	
	UpPopup(ActionMenuPopup);
	Display(ActionMenu_1,
		"Select action for:\xff\xc0");

	strcpy(selname,selfile -> name);
	if (selfile -> ext[0]) 
	{
		strcat(selname,".");
		strcat(selname,selfile -> ext);
	}
	strcat(selname,"\xff\xc0");
	
	Display(ActionMenu_2,selname);	/* display filename */

	MenuMount();	/* display the action-select menu */

	return(1);
	
}


di()	/* service for file-action select */

{

char selname[50];
char linebuf[100];
int linelen;
int  printer;
char fname[80];

	while (1) 
	{
	
		kpfunret = MenuInput(ipdir);  /* get menu selection */
		
		if (kpfunret >= -1)
			break;	/* values < -1 are PF keys; not used in this menu */
	
	}
	
	if (kpfunret == -1) 	/* ESC */
	{
		UnstackAction();    	/* TextLib -- unstack this action */
		return(1);			/* return to main loop */
	}
	
	/*	kpfunret is positive */
	
	switch (MenuSelection(dirmenu)) 	/* user's selection */
	{
			
	case	FileDelete:

		strcpy(file2,pathpart);
		strcat(file2,selfile -> name);
		if (selfile -> ext[0]) 
		{
			strcat(file2,".");
			strcat(file2,selfile -> ext);
		}

		strcpy(fname,"Do you really want to delete ");
		strcat(fname,file2);
		conflag = confirm(fname);
		delfile();
		break;
		
	case	FileCopy:
	
		StackAction("ctofile"); /* get to-file and do the copy */
		break;
		
	case	FilePrint:
	
		strcpy(selname,pathpart);
		strcat(selname,selfile -> name);
		if (selfile -> ext[0]) 
		{
			strcat(selname,".");
			strcat(selname,selfile -> ext);
		}

		if (! (fd1 = fopen(selname,"rt"))) 
		{
			FastMsg("a","Error opening file");
			break;
		}

		if ((printer = open("PRN",O_WRONLY | O_TEXT)) == -1) 
		{
			FastMsg("a","Error starting the printout");
			break;
		}	
		
		criterr = 0;

		/* \xff\x82 is a Blink video-attribute modifier */

		Display(StatusWindow_1,"\xff\x82 Printing...\xff\xc0");
			
		sprintf(linebuf,"\nLISTING OF: %s\n\n",selname);
		putpr(linebuf,printer);
			
		while (1)  	/* print the file */
		{
			if (! fgets(linebuf,81,fd1))
				break;
				
			linelen = strlen(linebuf);
				
			if (linebuf[linelen - 1] != '\n') 
			{
				linebuf[linelen] = '\n';
				linebuf[linelen + 1] = 0;
			}
			
			putpr(linebuf,printer);
			
		}

		closeprt(printer);
		
		Clear(StatusWindow_1);

		break;
		
	case	FileRename:
	
		StackAction("rtofile"); /* get new name and do rename */
		break;
		
	case	FileBrowse:
	
		StackAction("browse");
		break;
		
	case	FileEdit:

		strcpy(edfile_name,selfile -> name);
		strcpy(edfile_ext,selfile -> ext);
		
		if (! strcmpi(selfile -> ext, "BAK"))
		{
			FastMsg("a","Not allowed to edit file w/ extension .BAK");
			break;
		}

		StackAction("edit");
		break;
	
	case	FileSeeInfo:

		/*
		We're going to bring another popup right on top of
		ourselves.  Since no input is required (other than
		"press any key to continue"), it is not necessary
		to define an Action for this new popup.  When the
		user has finished viewing the file information,
		we will take down the new popup, thus re-exposing
		the popup we are currently on.  This stacking of
		popups is a built-in TextLib function.
		*/
		
		UpPopup(FileInfoPopup);

		Display(FileInfo_1,"File information for:\xff\xc0");

		strcpy(selname,selfile -> name); /* file-name */
		if (selfile -> ext[0]) 
		{
			strcat(selname,".");
			strcat(selname,selfile -> ext);
		}
		strcat(selname,"\xff\xc0");
		
		Display(FileInfo_2,selname);	
		
		Display(FileInfo_3,(char *)&fileihdr);	/* file info header */

		getfldat();		/* extract file-date info */

		fmtdt();		/* format file date */
		movmem(wkarea,fileinfo.date,8);
		fmttm();		/* file time */
		movmem(wkarea,fileinfo.time,6);
		fmtsize();		/* file size */
		movmem(wkarea,fileinfo.size,8);
		
		fileinfo.arc = (selfile -> attr) & 0x20 ? 'Y' : 'N';
		fileinfo.rdonly = (selfile -> attr) & 0x01 ? 'Y' : 'N';
		
		Display(FileInfo_4,(char *)&fileinfo); /* display info on screen */
		
		Display(FileInfo_5,
				"press any key to continue...\xff\xc0");
				
		if (! getch())/* wait for user's keypress */
			getch();
		
		DownPopup(FileInfoPopup);	/* Take down the info popup */
		
		break;
		
	}

	return(1);
	
}

diexit()	/* menu exit */

{

	DownPopup(ActionMenuPopup); /* take down the menu popup */
	return(1);
	
}



















